/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.modeling.ui.diagramEditor;

import java.util.Collections;
import java.util.Set;

import org.simantics.db.common.request.ParametrizedRead;
import org.simantics.db.layer0.request.combinations.Combinators;
import org.simantics.diagram.participant.PointerInteractor2;
import org.simantics.g2d.canvas.ICanvasContext;
import org.simantics.g2d.diagram.DiagramHints;
import org.simantics.g2d.diagram.IDiagram;
import org.simantics.g2d.diagram.participant.pointertool.PointerInteractor;
import org.simantics.g2d.layers.ILayer;
import org.simantics.g2d.layers.ILayers;
import org.simantics.g2d.layers.ILayersEditor;
import org.simantics.modeling.ui.diagramEditor.handlers.StructuralBrowsingHandler;
import org.simantics.ui.workbench.IResourceEditorInput;
import org.simantics.ui.workbench.editor.input.InputValidationCombinators;

/**
 * @author Tuukka Lehtonen
 */
public class PlainDiagramViewer extends org.simantics.modeling.ui.diagramEditor.DiagramViewer {

    ParametrizedRead<IResourceEditorInput, Boolean> INPUT_VALIDATOR =
        Combinators.compose(
        		InputValidationCombinators.hasURI(),
                InputValidationCombinators.extractInputResource()
        );

    protected String getPopupId() {
        return "#PlainDiagramViewerPopup";
    }
    
    public ParametrizedRead<IResourceEditorInput, Boolean> getInputValidator() {
        return INPUT_VALIDATOR;
    }

    @Override
    protected PointerInteractor getPointerInteractor() {
        return new PointerInteractor2(true, true, true, false, false, false, synchronizer.getElementClassProvider());
    }
    
    protected void addStructureParticipants(ICanvasContext ctx) {
        ctx.add(new StructuralBrowsingHandler(getSite(), sessionContext, getResourceInput2()));
        addWorkbenchSelectionProvider(ctx);

        ctx.add(
                new TerminalInformer(
                        c,
                        resourceManager,
                        getEditorSite().getActionBars().getStatusLineManager(),
                        new DefaultTerminalNamingStrategy()
                        )
                );
    }

    @Override
    protected void beforeSetDiagram(IDiagram diagram) {
        setupLayers(diagram);
    }

    protected void setupLayers(IDiagram diagram) {
        diagram.setHint(DiagramHints.KEY_LAYERS, new ILayers() {

            @Override
            public boolean isActive(ILayer layer) {
                return true;
            }

            @Override
            public Set<ILayer> getVisibleLayers() {
                return Collections.emptySet();
            }

            @Override
            public Set<ILayer> getLayers() {
                return Collections.emptySet();
            }

            @Override
            public boolean getIgnoreVisibilitySettings() {
                return true;
            }

            @Override
            public boolean getIgnoreFocusSettings() {
                return true;
            }

        });

        // Disable roles (layers) by default in symbol editor to show everything.
        ILayersEditor le = diagram.getHint(DiagramHints.KEY_LAYERS_EDITOR);
        if (le != null) {
            le.setIgnoreFocusSettings(true);
            le.setIgnoreVisibilitySettings(true);
        }
    }

    @Override
    protected EditorState getSavedEditorState(ICanvasContext ctx) {
        return DiagramEditorStates.toEditorState(ctx, true, true, false);
    }

}
